##############################################################################
#
#    Xbus Consumer/Worker, for Odoo
#    Copyright © 2013, 2022, 2023, 2024 XCG Consulting <https://xcg-consulting.fr/>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import logging
from enum import Enum
from typing import Literal

from odoo import _, api, exceptions, models  # type: ignore[import-untyped]

_logger = logging.getLogger(__name__)


class ActorType(str, Enum):
    """Types of actor"""

    CONSUMER = "consumer"
    WORKER = "worker"


class UnprocessedEnvelopeException(Exception):
    """Exception when the received envelope was not processed, either something that
    should not have been received, missing code or misconfiguration."""


class XDataLoader(models.AbstractModel):
    """Abstract model to have the consumer/worker process method used by xbus-odoo."""

    _name = "x_data_loader"
    _description = "External Data Loader"

    # This method is called in RPC by xbus-odoo
    @api.model
    def process(
        self,
        actor_type: ActorType,
        actor: str,
        tag: str,
        inputs: list[dict[str, int | str]],
    ) -> Literal[True] | list[dict[str, int | str]]:
        """Process the incoming envelopes for the provided actor type/actor/tag.

        :param actor_type: consumer/worker
        :param actor: name of the actor. do not put logic based on this field, use tag
        instead
        :param tag: tag of the actor.
        :param inputs: envelope id by input name
        :return: outputs, empty dict if actor_type is consumer, envelope id by output
        name if actor_type is worker
        """
        _logger.info("Process inputs for %s (actor %s)", tag, actor)
        # Add a key showing the fact this loading is external.
        model = self.with_context(external_load=True)
        envelope_model = self.env["xbus.envelope"]
        # Transform input into something more useful to us
        inputs_dict = {
            input["Name"]: envelope_model.browse(input["EnvelopeID"])
            for input in inputs
        }
        match actor_type:
            case ActorType.CONSUMER:
                model.process_consumer(tag, inputs_dict)
                # xbus-odoo expects True for consumers
                return True
            case ActorType.WORKER:
                raw_outputs = model.process_worker(tag, inputs_dict)
                return [
                    {"Name": key, "EnvelopeID": value.id if value else 0}
                    for key, value in raw_outputs.items()
                ]
        # Also return true if called with an unknown actor type
        raise exceptions.UserError(_("Invalid Actor Type"))

    @api.model
    def process_consumer(self, tag: str, inputs: dict[str, models.BaseModel]):
        """Process consumer envelopes.
        Implement to process those envelopes.

        :param tag: tag of the actor.
        :param inputs: envelope by input name
        :rtype: None
        """
        raise UnprocessedEnvelopeException  # pragma: no cover

    @api.model
    def process_worker(
        self, tag: str, inputs: dict[str, models.BaseModel]
    ) -> dict[str, models.BaseModel]:
        """Process worker envelopes.
        Implement to process those envelopes.

        :param tag: tag of the actor.
        :param inputs: envelope by input name
        :return: envelope by output name
        """
        raise UnprocessedEnvelopeException  # pragma: no cover
