##############################################################################
#
#    Xbus Consumer/Worker, for Odoo
#    Copyright © 2013 XCG Consulting <https://www.xcg-consulting.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import json
import logging
from collections import defaultdict
from enum import Enum

from odoo import Command, api, models  # type: ignore[import-untyped]

_logger = logging.getLogger(__name__)


class Tag(str, Enum):
    TEST_CONSUMER = "test-consumer"


class Input(str, Enum):
    DEFAULT = "default"


class Output(str, Enum):
    DEFAULT = "default"


class XDataLoader(models.AbstractModel):
    """This implementation is used in tests and is an example implementation that would
    be in another module."""

    _inherit = "x_data_loader"

    @api.model
    def process_consumer(self, tag: str, inputs: dict[str, models.Model]):
        # Our documentation for setting up xbus-odoo with our module would indicate to:
        # - set the tag to test-consumer when setting up the account
        # - the input should be default
        # - the message type is test_1
        partners_to_create = []
        match tag:
            case "test-consumer":
                for input_name in inputs:
                    match input_name:
                        case Input.DEFAULT:
                            for message in inputs[input_name].message_ids:
                                match message.type:
                                    case "test_1":
                                        partners_to_create.append(
                                            json.loads(message.body.decode("UTF-8"))
                                        )
                                    case _:  # pragma: no cover
                                        _logger.error(
                                            "Unexpected message type %s in input %s "
                                            "(tag %s)",
                                            message.type,
                                            input_name,
                                            tag,
                                        )
                        case _:  # pragma: no cover
                            _logger.error(
                                "Unexpected input %s (tag %s)", input_name, tag
                            )
            case _:  # pragma: no cover
                _logger.error("Unexpected tag %s", tag)
        # do all partners creation in one pass
        if partners_to_create:
            self.env["res.partner"].create(partners_to_create)

    @api.model
    def process_worker(
        self, tag: str, inputs: dict[str, models.BaseModel]
    ) -> dict[str, models.BaseModel]:
        # This code expects a single creation message. Otherwise, the output would need
        # to contain all the ids
        messages = defaultdict(list)
        match tag:
            case "test-worker":
                for input_name in inputs:
                    match input_name:
                        case Input.DEFAULT:
                            for message in inputs[input_name].message_ids:
                                match message.type:
                                    case "test_1":
                                        new_partner = self.env["res.partner"].create(
                                            [json.loads(message.body.decode("UTF-8"))]
                                        )
                                        # Real code would return an XML id.
                                        messages[Output.DEFAULT].append(
                                            Command.create(
                                                {
                                                    "type": "test_2",
                                                    "body": b'{"id": %d}'
                                                    % new_partner.id,
                                                }
                                            )
                                        )
                                    case _:  # pragma: no cover
                                        _logger.error(
                                            "Unexpected message type %s in input %s "
                                            "(tag %s)",
                                            message.type,
                                            input_name,
                                            tag,
                                        )
                        case _:  # pragma: no cover
                            _logger.error(
                                "Unexpected input %s (tag %s)", input_name, tag
                            )
            case _:  # pragma: no cover
                _logger.error("Unexpected tag %s", tag)
        # Could be improved to call a single create
        return {
            output: self.env["xbus.envelope"].create(
                [
                    {
                        "comment": "test",
                        "message_ids": messages[output],
                    }
                ]
            )
            for output in messages
        }
