##############################################################################
#
#    Xbus Consumer/Worker, for Odoo
#    Copyright © 2013, 2022, 2023, 2024 XCG Consulting <https://xcg-consulting.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import json
import uuid

from odoo import models  # type: ignore[import-untyped]
from odoo.addons.xbus_common.tests.modelcase import ModelCase
from odoo.exceptions import UserError  # type: ignore[import-untyped]


class Test(ModelCase):
    @classmethod
    def import_test_models(cls) -> list[models.Model]:
        result = super().import_test_models()
        from .models import XDataLoader  # pylint: disable=import-outside-toplevel

        result.append(XDataLoader)
        return result

    def setUp(self):
        super().setUp()
        # In real case, this is done by an external process
        uid = str(uuid.uuid4())
        self.cr.execute(
            "INSERT INTO xbus_envelope (comment, uid) VALUES ('consumer test-consumer',"
            " %s) RETURNING id",
            (uid,),
        )
        self.envelope_id = self.cr.fetchall()[0][0]
        uid = str(uuid.uuid4())
        self.cr.execute(
            "INSERT INTO xbus_envelope (comment, uid) VALUES ('consumer test-consumer',"
            " %s) RETURNING id",
            (uid,),
        )
        self.envelope_2_id = self.cr.fetchall()[0][0]
        uid = str(uuid.uuid4())
        self.cr.execute(
            "INSERT INTO xbus_message (envelope_id, uid, type, header, body) VALUES "
            "(%s, %s, %s, %s, %s) ",
            (
                self.envelope_id,
                uid,
                "test_1",
                None,
                b'{"name": "New Partner \\ud83d\\ude04"}',
            ),
        )

    def test_process(self):
        """Call :py:meth:~odoo.addons.x_data_loader.models.x_data_loader.XDataLoader.
        process with valid data.
        """
        # We are going to create resources.
        prev_resources = self.env["res.partner"].search([])

        self.env["x_data_loader"].process(
            "consumer",
            "actor-random-name",
            "test-consumer",
            [{"Name": "default", "EnvelopeID": self.envelope_2_id}],
        )
        new_resources = self.env["res.partner"].search([])
        self.assertEqual(len(new_resources), len(prev_resources))

        self.env["x_data_loader"].process(
            "consumer",
            "actor-random-name",
            "test-consumer",
            [{"Name": "default", "EnvelopeID": self.envelope_id}],
        )
        new_resources = self.env["res.partner"].search([])
        self.assertEqual(len(new_resources), len(prev_resources) + 1)
        new_partner = new_resources - prev_resources
        self.assertEqual(new_partner.name, "New Partner 😄")

    def test_process_worker(self):
        """Call :py:meth:~odoo.addons.x_data_loader.models.x_data_loader.XDataLoader.
        process with valid data.
        """
        # We are going to create resources.
        prev_resources = self.env["res.partner"].search([])

        result = self.env["x_data_loader"].process(
            "worker",
            "actor-random-name",
            "test-worker",
            [{"Name": "default", "EnvelopeID": self.envelope_2_id}],
        )
        self.assertFalse(result)
        new_resources = self.env["res.partner"].search([])
        self.assertEqual(len(new_resources), len(prev_resources))

        result = self.env["x_data_loader"].process(
            "worker",
            "actor-random-name",
            "test-worker",
            [{"Name": "default", "EnvelopeID": self.envelope_id}],
        )
        new_resources = self.env["res.partner"].search([])
        self.assertEqual(len(new_resources), len(prev_resources) + 1)
        new_partner = new_resources - prev_resources
        self.assertEqual(new_partner.name, "New Partner 😄")
        self.assertEqual(result[0]["Name"], "default")
        self.assertEqual(
            json.loads(
                self.env["xbus.envelope"]
                .browse(result[0]["EnvelopeID"])
                .message_ids.body
            )["id"],
            new_partner.id,
        )

    def test_process_other(self):
        with self.assertRaises(UserError):
            self.env["x_data_loader"].process(
                "other",
                "actor-random-name",
                "test-other",
                [{"Name": "default", "EnvelopeID": self.envelope_2_id}],
            )
